#!/bin/bash
# ==============================================================================
# LOGGING
# Example:
# >cat binary.sh
# source path.conf
# source ${common_var_dir}/libs/utils.sh
# init_logger "${userful_etc}/log.conf"
# # after initialization you can log stuff using 
# # log <level> <format> [args]
# # possible levels are: CRITICAL ERROR WARNING INFO DEBUG
# log DEBUG "TESTER Found '%i' monitors on '%s' videocard." 10 hello
# 
# ------------------------------------------------------------------------------

function read_config() {
	# Reading bash sourceable configuration file provided as second argument
	# Cycle through all variables defined there and add 'prefix' specified as
	# first argument to every variable name. Export those variables.
	# Example:
	# >cat log.conf
    # LOG_LEVEL=DEBUG
	# LOG_LEVEL_frontend=ERROR
	# >read_config "LOGGING_" "log.conf"
	# echo $LOGGING_LOG_LEVEL
	# DEBUG
	local prefix="$1"
	local filename="$2"
	local line=
	local key=
	local value=
	local separator==
	while read line; do
		if [ "${line%\#*}" == "" ] ; then # skip comment lines 
			continue
		fi
		key="${line%${separator}*}"
		value="${line#*${separator}}"
		unset ${prefix}${key}
		if [ "${prefix}" != "" -a "${key}" != "" -a "${value}" != "" ] ; then
			export ${prefix}${key}="${value}"
		fi
    done < ${filename}
}

function get_level_idx() {
	local level=$1
	local levels=( CRITICAL ERROR WARNING INFO DEBUG )
	local index=
	local length="${#levels[@]}"
	for (( index=0;index<${length};index++ )); do
		if [ "${levels[$index]}" == "${level}" ] ; then
			echo ${index}
		fi
	done
}

function get_program_name() {
	if [ -z "${BASH_SOURCE[1]}" ] ; then
		echo $(basename ${BASH_SOURCE[1]}) 
		return 0
	fi
	if [ "$0" == "-bash" ] ; then
		echo $(basename ${BASH_ARGV[0]})
		return 0
	fi
	echo $(basename $0)
}

###############################
# Arguments:
#   1: the name of configuration file
# Defines Globals:
#   PROGRAM_NAME
# Modifies Globals:
#   LOGGING_LOG_FILE
#   LOGGING_LOG_LEVEL
# Unset Globals:
#   LOGGING_LOG_FILE_PROGRAM
#   LOGGING_LOG_LEVEL_PROGRAM
###############################
function init_logger() {
	local config_file="$1"
	unset LOGGING_LOG_FILE
    unset LOGGING_LOG_FILE_PROGRAM
    unset LOGGING_LOG_LEVEL
    unset LOGGING_LOG_LEVEL_PROGRAM
    # Depends on Globals:
    #   LOGGING_LOG_FILE
    #   LOGGING_LOG_FILE_PROGRAM
    #   LOGGING_LOG_LEVEL
    #   LOGGING_LOG_LEVEL_PROGRAM
	read_config "LOGGING_" "${config_file}"
	export PROGRAM_NAME=$(get_program_name)

    # GETTING LOG FILE NAME
	LOGGING_LOG_FILE_PROGRAM="LOGGING_LOG_FILE_${PROGRAM_NAME%.*}" # form variable name
	eval LOGGING_LOG_FILE_PROGRAM=\$$LOGGING_LOG_FILE_PROGRAM    # getting value
	if [ -z "$LOGGING_LOG_FILE_PROGRAM" ] ; then
		LOGGING_LOG_FILE="$LOGGING_LOG_FILE"
	else
		LOGGING_LOG_FILE="$LOGGING_LOG_FILE_PROGRAM"
		unset LOGGING_LOG_FILE_PROGRAM
	fi
	LOGGING_LOG_FILE="${root}${LOGGING_LOG_FILE}"

    # GETTING LOG LEVEL
	LOGGING_LOG_LEVEL_PROGRAM="LOGGING_LOG_LEVEL_${PROGRAM_NAME%.*}" # form variable name
	eval LOGGING_LOG_LEVEL_PROGRAM=\$$LOGGING_LOG_LEVEL_PROGRAM      # getting value
	if [ -z "$LOGGING_LOG_LEVEL_PROGRAM" ] ; then
		LOGGING_LOG_LEVEL="$LOGGING_LOG_LEVEL"
	else
		LOGGING_LOG_LEVEL="$LOGGING_LOG_LEVEL_PROGRAM"
		unset LOGGING_LOG_LEVEL_PROGRAM
	fi	
}

################################
# log()
# Required variable logfile for now
# The function will work only if invoked in the same process
# Depends on Globals:
#   LOGGING_LOG_LEVEL
#   LOGGING_LOG_FORMAT_bash
#   LOGGING_LOG_FILE
#   PROGRAM_NAME
# Example:
#   log DEBUG "Found %i/%i elements." 10 15
#  as a result the following string would be written into log file
#   "2010-06-11 16:17:16:DEBUG:utils.sh:tester:157:"Found 10/15 elements.
################################
function log() {
		local level="$1"
		local level_idx=
        local program=$PROGRAM_NAME

		local program_level_idx=
		level_idx=$(get_level_idx "$level")
		program_level_idx=$(get_level_idx "${LOGGING_LOG_LEVEL}")
		if [ ${level_idx} -gt ${program_level_idx} ] ; then 
			return  1	# verbosity is less then needed
		fi		

        local lineno=${BASH_LINENO[0]}
        local func=${FUNCNAME[1]}
		local ts=`date +"%Y-%m-%d %T"`
        local format="$LOGGING_LOG_FORMAT_bash$2\n" # ts:program:caller:lineno:REST
        shift # skip level
		shift # skip format
        local values=( "$@" )   # quote arguments

		# get_logfile_name
        printf "$format" "$ts" "$level" "$program" "$func" "${lineno}" "${values[@]}" \
			>> $LOGGING_LOG_FILE
}

# test function used from test suite
function tester(){
	log DEBUG "TESTER Found '%i' monitors on '%s' videocard." 10 hello
}

# ==============================================================================
